"use strict";
exports.__esModule = true;
exports.getResReqTypes = exports.formatDescription = exports.resolveValue = exports.getScalar = exports.isReference = exports.getDocs = void 0;
var lodash_1 = require("lodash");
var uniq = lodash_1["default"].uniq, isEmpty = lodash_1["default"].isEmpty;
var case_1 = require("case");
var pascal = case_1["default"].pascal;
var getDocs = function (data) {
    return (0, exports.isReference)(data) ? '' : (0, exports.formatDescription)(data.description);
};
exports.getDocs = getDocs;
/**
 * Discriminator helper for `ReferenceObject`
 */
var isReference = function (property) {
    return Boolean(property.$ref);
};
exports.isReference = isReference;
/**
 * Return the typescript equivalent of open-api data type
 */
var getScalar = function (item) {
    var nullable = item.nullable ? ' | null' : '';
    switch (item.type) {
        case 'number':
        case 'integer':
            return 'number' + nullable;
        case 'boolean':
            return 'boolean' + nullable;
        case 'array':
            return getArray(item) + nullable;
        case 'string':
            return (item["enum"] ? "\"".concat(item["enum"].join("\" | \""), "\"") : 'string') + nullable;
        case 'object':
        default:
            return getObject(item) + nullable;
    }
};
exports.getScalar = getScalar;
/**
 * Return the output type from the $ref
 */
var getRef = function ($ref) {
    if ($ref.startsWith('#/components/schemas')) {
        return pascal($ref.replace('#/components/schemas/', ''));
    }
    else if ($ref.startsWith('#/components/responses')) {
        return pascal($ref.replace('#/components/responses/', '')) + 'Response';
    }
    else if ($ref.startsWith('#/components/parameters')) {
        return pascal($ref.replace('#/components/parameters/', '')) + 'Parameter';
    }
    else if ($ref.startsWith('#/components/requestBodies')) {
        return pascal($ref.replace('#/components/requestBodies/', '')) + 'RequestBody';
    }
    else {
        throw new Error('This library only resolve $ref that are include into `#/components/*` for now');
    }
};
/**
 * Return the output type from an array
 */
var getArray = function (item) {
    if (item.items) {
        if (!(0, exports.isReference)(item.items) && (item.items.oneOf || item.items.allOf || item.items["enum"])) {
            return "(".concat((0, exports.resolveValue)(item.items), ")[]");
        }
        else {
            return "".concat((0, exports.resolveValue)(item.items), "[]");
        }
    }
    else {
        throw new Error('All arrays must have an `items` key define');
    }
};
/**
 * Return the output type from an object
 */
var getObject = function (item) {
    if ((0, exports.isReference)(item)) {
        return getRef(item.$ref);
    }
    if (item.allOf) {
        return item.allOf.map(exports.resolveValue).join(' & ');
    }
    if (item.oneOf) {
        return item.oneOf.map(exports.resolveValue).join(' | ');
    }
    if (!item.type && !item.properties && !item.additionalProperties) {
        return '{}';
    }
    // Free form object (https://swagger.io/docs/specification/data-models/data-types/#free-form)
    if (item.type === 'object' &&
        !item.properties &&
        (!item.additionalProperties || item.additionalProperties === true || isEmpty(item.additionalProperties))) {
        return '{[key: string]: any}';
    }
    var IdentifierRegexp = /^[a-zA-Z_$][a-zA-Z0-9_$]*$/;
    // Consolidation of item.properties & item.additionalProperties
    var output = '{\n';
    if (item.properties) {
        output += Object.entries(item.properties)
            .map(function (_a) {
            var key = _a[0], prop = _a[1];
            var doc = (0, exports.getDocs)(prop);
            var isRequired = (item.required || []).includes(key);
            var processedKey = IdentifierRegexp.test(key) ? key : "\"".concat(key, "\"");
            return "".concat(doc, "\n").concat(processedKey).concat(isRequired ? '' : '?', ": ").concat((0, exports.resolveValue)(prop), ";");
        })
            .join('');
    }
    if (item.additionalProperties) {
        if (item.properties) {
            output += '\n';
        }
        output += "} & { [key: string]: ".concat(item.additionalProperties === true ? 'any' : (0, exports.resolveValue)(item.additionalProperties));
    }
    if (item.properties || item.additionalProperties) {
        if (output === '{\n') {
            return '{}';
        }
        return output + '\n}';
    }
    return item.type === 'object' ? '{[key: string]: any}' : 'any';
};
/**
 * Resolve the value of a schema object to a proper type definition.
 */
var resolveValue = function (schema) {
    return (0, exports.isReference)(schema) ? getRef(schema.$ref) : (0, exports.getScalar)(schema);
};
exports.resolveValue = resolveValue;
/**
 * Format a description to code documentation.
 */
var formatDescription = function (description) {
    if (!description) {
        return '';
    }
    return "\n/**\n".concat(description
        .split('\n')
        .map(function (i) { return "*  ".concat(i); })
        .join('\n'), "\n */");
};
exports.formatDescription = formatDescription;
/**
 * Extract responses / request types from open-api specs
 */
var getResReqTypes = function (responsesOrRequests) {
    return uniq(responsesOrRequests.map(function (_a) {
        var _ = _a[0], res = _a[1];
        if (!res) {
            return;
        }
        if ((0, exports.isReference)(res)) {
            return getRef(res.$ref);
        }
        if (res.content) {
            for (var _i = 0, _b = Object.keys(res.content); _i < _b.length; _i++) {
                var contentType = _b[_i];
                if (contentType.startsWith('application/json') ||
                    contentType.startsWith('application/octet-stream')) {
                    var schema = res.content[contentType].schema;
                    return (0, exports.resolveValue)(schema);
                }
            }
            return;
        }
        return;
    })).join(' | ');
};
exports.getResReqTypes = getResReqTypes;
